//
// Copyright (C) 1999-2004 WideStudio Development Team
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to
// deal in the Software without restriction, including without limitation the
// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
// sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.
// Except as contained in this notice, the name of WideStudio Development Team 
// shall not be used in advertising or otherwise to promote the sale, use or 
// other dealings in this Software without prior written authorization from  
// WideStudio Development Team.

#include  <stdlib.h>
#include  "axis.h"

TAxis::TAxis()
{
    change = 1;
    origin = Vector(0,0,0);
    SetAngle(0, 0, 0);
}

void TAxis::move(Vector v)
{

    v = ConvCoordinate(m, v);
    origin = origin + v;
    change = 1;
}

void TAxis::rotate(long h, long p, long b)
{
    EulerAngle(h, p, b);
    CalcRotationMatrix();
    change = 1;
}

void TAxis::set_origin(long x, long y, long z)
{
    origin = Vector(x, y, z);
    change = 1;
}

Vector TAxis::get_origin()
{
    return origin;
}

Matrix3 TAxis::get_matrix()
{
    CalcRotationMatrix();
    return m;
}

void TAxis::set_matrix(Matrix3 tm)
{
    m = tm;
}

Quaternion TAxis::get_quat()
{
    return q;
}

Vector TAxis::RelativePosition(Vector v)
{
    return ConvCoordinate2(m, v);
}

void TAxis::attitude(long h, long p, long b)
{
    SetAngle(h, p, b);
    CalcRotationMatrix();
    change = 1;
}

void TAxis::get_attitude(long &h, long &p, long &b)
{
    GetAngle(h,p,b);
}

void TAxis::CalcRotationMatrix()
{
    QuatToMat(q, m);
}

void TAxis::SetAngle(long h, long p, long b)
{
    EularToQuat(h, p, b, q);
}

void TAxis::GetAngle(long &h, long &p, long &b)
{
    CalcRotationMatrix();
    b  = DEG( atan2(m.m8, m.m9) );
    p  = DEG( - asin(m.m7) );
    h  = DEG( atan2(m.m4, m.m1) );
}

void TAxis::EulerAngle(long DH, long DP, long DB)
{
    Quaternion nq;

    long M = DH;
    if (abs(M) < abs(DP)) M = DP;
    if (abs(M) < abs(DB)) M = DB;
    M = M / 64;
    if (M < 0) M = -M;
    M++;
    double tDH = RAD(DH) / M;
    double tDP = RAD(DP) / M;
    double tDB = RAD(DB) / M;
    for(int n = 1; n<=M; n++) {
        double AL = tDB*tDB + tDP*tDP + tDH*tDH;
        double D1 = (AL*tDB )/48.0;
        double D2 = (AL*tDP )/48.0;
        double D3 = (AL*tDH )/48.0;

        double AQ = 1.0 - 0.125 * AL;
        double BQ = -0.5 * tDB + D1;
        double CQ = -0.5 * tDP + D2;
        double DQ = -0.5 * tDH + D3;

        nq._0 =  AQ*q._0 + BQ*q._1 + CQ*q._2 + DQ*q._3;
        nq._1 = -BQ*q._0 + AQ*q._1 - DQ*q._2 + CQ*q._3;
        nq._2 = -CQ*q._0 + DQ*q._1 + AQ*q._2 - BQ*q._3;
        nq._3 = -DQ*q._0 - CQ*q._1 + BQ*q._2 + AQ*q._3;
        q = nq;
    }
}

void TAxis::attitudeq(Quaternion qq)
{

    q = qq;
    CalcRotationMatrix();
    change = 1;
}

